/* REXX */
/*
* copyright-start
**********************************************************************
*
*  LICENSED MATERIALS - PROPERTY OF IBM
*  5724-V27
*  Copyright IBM Corp. 2016
*
*  US GOVERNMENT USERS RESTRICTED RIGHTS - USE, DUPLICATION OR
*  DISCLOSURE RESTRICTED BY GSA ADP SCHEDULE CONTRACT WITH
*  IBM CORP.
*
**********************************************************************
* copyright-end
*
*  REXX for invoking DROP PACKAGE commands for obsolete packages for 
*  Rational the Asset Analyzer Universal Data Base (UDB) image.
*
*  REXX dmhDropPackages.rexx <inDbName> <inSchema> <inLogFile>
*
*    where:
*
*       inDbName = database name
*       inSchema = qualifier for collection of database tables
*       logFile  = file to receive output
*
*  The steps involved are:
*
*    1. Load SQL interface
*    2. Connect to the database
*    3. Fetch RAA packages from system catalog and generate DROP PACKAGE commands
*    4. Write DROP PACKAGE commands to <tempDir>\dmhDropPackages.sql
*    4. Execute the DROP PACKAGE commands in <tempDir>\dmhDropPackages.sql
*    5. Disconnect from the database
*
**********************************************************************
*/
parse arg inDbName inSchema logFile tempDir

StartTime = time()

DropPackagesFile = tempDir"\dmhDropPackages.sql"
'@del' DropPackagesFile '>nul 2>nul'

OutFile = logFile
OutFile = strip(OutFile, 'B', '"')
if OutFile = '' then
   OutFile = "..\log\dmhDropPackages.log"
'@del "'OutFile'" >nul 2>nul'

call SetupDB2

if inDbName = '' then
   do
     say "Using database DMHDB"
     dbName = 'DMHDB'
   end
else
   dbName = inDbName

if inSchema = '' then
   do
     say "Using schema DMH"
     sName = 'DMH'
   end
else
   sName = inSchema

SqlStmt = "Connect to" dbName
call SQLEXEC SqlStmt
if ( RESULT <> 0 ) then
   call DB2Error SqlStmt
say "Connected to database"

call PrepareDROPPACKAGECommands
call DoSQLEXEC "OPEN c1"
say "Writing DROP PACKAGE commands..."
call lineout DropPackagesFile, "CONNECT TO" dbName";"
do forever
   call SQLEXEC "FETCH c1 INTO :DropPackagesSql"
   if ( SQLCA.SQLCODE = 0 ) then
      do
           call lineout DropPackagesFile, DropPackagesSql
      end
   else
      if ( SQLCA.SQLCODE = 100 ) then
         leave
      else
         call DB2Error "FETCH c1"
end
call lineout DropPackagesFile
call lineout DropPackagesFile, "CONNECT RESET;"
call DoSQLEXEC "CLOSE c1"
call DoDropPackages
call TearDownDB2
call lineout OutFile

StopTime = time()
say "Timing" StartTime StopTime
say "DROP PACKAGE processing completed"
exit

DoDropPackages:
   Say "Running DROP PACKAGE commands..."
   "db2cmd /c /w /i  db2 -v -td; -f "DropPackagesFile" >" Outfile
   return

PrepareDROPPACKAGECommands:
   SqlStmt = "SELECT 'DROP PACKAGE '||packageName||' FROM SYSCAT.PACKAGES;' FROM(SELECT RTRIM(pkgschema)||'.'||RTRIM(pkgname) as packageName FROM syscat.packages where UPPER(pkgschema) = '"TRANSLATE(sName)"' AND pkgname like 'DMHD%' ) p ORDER BY packageName"
   call DoSQLEXEC "PREPARE s1 FROM :SqlStmt"
   call DoSQLEXEC "DECLARE c1 CURSOR FOR s1"
   return

DoSQLDBS:
   parse arg Stmt
   call SQLDBS Stmt
   if ( RESULT <> 0 ) then
      call DB2Error Stmt
   if ( SQLCA.SQLCODE <> 0 ) then
      call DB2Error Stmt
   return

DoSQLEXEC:
   parse arg Stmt
   call SQLEXEC Stmt
   if ( RESULT <> 0 ) then
      call DB2Error Stmt
   if ( SQLCA.SQLCODE <> 0 ) then
      call DB2Error Stmt
   return

SetupDB2:
   parse arg iDBName
   FunctionsToLoad = "SQLDBS SQLDB2 SQLEXEC"

   do while ( FunctionsToLoad <> "" )
      parse var FunctionsToLoad FunctionName FunctionsToLoad
      if ( RxFuncQuery( FunctionName ) = 0 ) then
         iterate
      RC = RxFuncAdd( FunctionName, "DB2AR", FunctionName )
      if ( RC <> 0 ) then
         call DB2Error "RxFuncAdd of" FunctionName || ", RC =" RC
      else
         say FunctionName "loaded"
   end

   say "SQL interface loaded"
   return

TeardownDB2:
   SqlStmt = "Connect reset"
   call SQLEXEC SqlStmt
   if ( RESULT <> 0 ) then
      call DB2Error SqlStmt
   say "Disconnected"
   return

DB2Error:
   parse arg Message
   say "DB2 Error:" Message
   say "SQLCODE:  " SQLCA.SQLCODE
   say "SQLMSG:   " SQLMSG
   call SQLEXEC "ROLLBACK"
   exit
